#include <stdio.h>
#include <stddef.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>
#include <signal.h>
#include <process.h>
#include <math.h>
#include <time.h>
#include <sys/name.h>
#include <sys/kernel.h>
#include <sys/proxy.h>
#include "ttdriver.h"
#include "valves.h"
#include "regnames.h"
#include "panthermsg.h"

FILE 	*fpGC = NULL;
FILE	*fpMS = NULL;
FILE	*fpPAN = NULL;

int switch_gsv( int valveID, int valvePos ) {
	char 	msg[3],
			Rmsg[3];
	pid_t	valve_pid;

	msg[0] = msg_gsvSwitch;
	msg[1] = valvePos;		// valve position (1 or 2)
	msg[2] = valveID;	
	
	valve_pid = qnx_name_locate (0, NAME_VALVE, 0, NULL);

	if (valve_pid == -1) {
//		printf("unable to find valve server pid =-1 \n");
		return 1;
	}
	
	Send( valve_pid, msg, Rmsg, sizeof(msg), sizeof(Rmsg) );

	return 0;
}

// Function switches both of the MS valves.
void switch_MSgsvs_both ( unsigned int I1orI2 ) {

	// 1 is load for a gsv, 2 is inject
	if ( I1orI2 == 1 ) {		// Inj 1
		switch_gsv( 7, 2 );		 
		switch_gsv( 8, 2 );		
	} else {					// Inj 2
		switch_gsv( 7, 1 );		
		switch_gsv( 8, 1 );		
	}
}

int switch_ssv( int valveID, int valvePos ) {
	char 	msg[3],
			Rmsg[3];
	pid_t	valve_pid;

	msg[0] = msg_ssvSwitch;
	msg[1] = valvePos;		// valve position (1 to 6)
	msg[2] = valveID;	
	
	valve_pid = qnx_name_locate (0, NAME_VALVE, 0, NULL);

	if (valve_pid == -1) {
//		printf("unable to find valve server pid =-1 \n");
		return 1;
	}

	Send( valve_pid, msg, Rmsg, sizeof(msg), sizeof(Rmsg) );

	return 0;
}

// Function to set four solenoids to a predefined state.  These solenoids act as the SSV for 
// the PAN channel.
int PANssvState( int state ) {

	static int oldstate;

	if ( state == 0 ) {
		return oldstate;
	}

	oldstate = state;

	switch ( state ) {
		case 1: 						// air sample
			tt_scdc_command( 4 );
			tt_scdc_command( 106 );
			break;
		case 2: 						// cal sample
			tt_scdc_command( 104 );
			tt_scdc_command( 106 );
			break;
		case 3: 						// blank
			tt_scdc_command( 4 );
			tt_scdc_command( 6 );
			break;
		default:
			tt_scdc_command( 4 );
			tt_scdc_command( 6 );
			break;
	}

	return state;

}


int handlessvGC( int valve_num, int cmd, int runmode ) {

	static int 	old_mode;
	int 	pos;
	char	ssv_table[20];

	// Do nothing command.
	if (cmd == 0)
		return 0;

	if (old_mode != runmode)
		fclose(fpGC);
	
	old_mode = runmode;

	if (runmode == 2)
		strcpy( ssv_table, "gcRun.vtbl" );
	else if (runmode == 7)
		strcpy( ssv_table, "gcCal.vtbl" );
	else
		return 0;

	pos = get_one_num( ssv_table, valve_num );
	switch_ssv( valve_num, pos );

	return 1;
}

int handlessvMS( int valve_num, int cmd, int runmode ) {

	static int 	old_mode;
	int 	pos;
	char	ssv_table[20];

	// Do nothing command.
	if (cmd == 0)
		return 0;

	if (old_mode != runmode)
		fclose(fpMS);
	
	old_mode = runmode;

	if (runmode == 2)
		strcpy( ssv_table, "msRun.vtbl" );
	else if (runmode == 3)
		strcpy( ssv_table, "msLab.vtbl" );
	else if (runmode == 7)
		strcpy( ssv_table, "msCal.vtbl" );
	else
		return 0;

	pos = get_one_num( ssv_table, valve_num );
	switch_ssv( valve_num, pos );

	return 1;
}

// valve_num should == 9 for PAN ssv solenoids
int handlessvPAN( int valve_num, int cmd, int runmode ) {

	static int 	old_mode;
	int 	state;
	char	ssv_table[20];

	// Do nothing command.
	if (cmd == 0)
		return 0;

	if (old_mode != runmode)
		fclose(fpPAN);
	
	old_mode = runmode;

	if (runmode == 2)
		strcpy( ssv_table, "panRun.vtbl" );
	else if (runmode == 7)
		strcpy( ssv_table, "panCal.vtbl" );
	else
		return 0;

	state = get_one_num( ssv_table, valve_num );
	PANssvState( state );

	return 1;
}


int get_one_char(char *table, int valve_num) {
	int c;

	if( valve_num == 1 ) {

		if (fpGC != NULL)
			c = fgetc(fpGC);
		else {
			fpGC = fopen( table, "r" ); 
			if ( fpGC == NULL ) {
				perror("Unable to open SSV1 (GC) valve table file.\n");
				exit(EXIT_FAILURE);
			}
			return -1;
		}		

		if ( c == EOF ) {
			fclose( fpGC );
			fpGC = fopen( table, "r" ); 
			if ( fpGC == NULL ) {
				perror("Unable to open SSV1 (GC) valve table file.\n");
				exit(EXIT_FAILURE);
			}
			return -1;
		}		

	} else if ( valve_num == 2 ) {

		if ( fpMS != NULL )
			c = fgetc( fpMS );
		else {
			fpMS = fopen( table, "r" ); 
			if ( fpMS == NULL ) {
				perror("Unable to open SSV2 (MS) valve table file.\n");
				exit(EXIT_FAILURE);
			}
			return -1;
		}		

	 	if ( c == EOF ) {
			fclose( fpMS );
			fpMS = fopen( table, "r" ); 
			if ( fpMS == NULL ) {
				perror("Unable to open SSV2 (MS) valve table file.\n");
				exit(EXIT_FAILURE);
			}
			return -1;
		}		

	} else if ( valve_num == 9 ) {			// dummy number for PAN ssv solenoids
		
		if ( fpPAN != NULL )
			c = fgetc( fpPAN );
		else {
			fpPAN = fopen( table, "r" ); 
			if ( fpPAN == NULL ) {
				perror("Unable to open PAN valve table file.\n");
				exit(EXIT_FAILURE);
			}
			return -1;
		}		

	 	if ( c == EOF ) {
			fclose( fpPAN );
			fpPAN = fopen( table, "r" ); 
			if ( fpPAN == NULL ) {
				perror("Unable to open PAN valve table file.\n");
				exit(EXIT_FAILURE);
			}
			return -1;
		}		

	} else {
		perror(" Bad SSV valve number.\n");
		exit(EXIT_FAILURE);
	}

	if ( ( c >= '0') && ( c <= '9' ) )
		return c - '0';

	return -1;
}	


int get_one_num( char *table, int valve_num ) {
	int num = -1;

	while( num == -1 ) 
		num = get_one_char( table, valve_num );

	return num;
}

